﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media.Imaging;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Extensibility;
using ESRI.ArcGIS.Client.Geometry;

namespace NavAddin
{

	// Convenient base class for navigation commands
	#region NavCommand
	public abstract class NavCommand : ICommand
	{

		private bool _bInitialized = false;
		private bool _bMapEnabled = false;

		#region Overridable Methods
		abstract protected bool NavEnabled();

		abstract protected void NavInitialized();

		abstract protected void NavExtentUpdated();

		abstract protected void NavExecute();
		#endregion

		#region ICommand Implementation
		public bool CanExecute(object parameter)
		{
			if (!_bInitialized)
			{
				_bInitialized = true;
				NavBehavior nb = NavBehavior.Current;
				nb.MapExtentUpdated += new EventHandler(nb_MapExtentUpdated);
				if (!nb.IsInitialized)
				{
					nb.Initialized += new EventHandler(nb_Initialized);
					return false;
				}
				_bMapEnabled = true;
			}
			if (!_bMapEnabled)
				return false;
			return this.NavEnabled();
		}

		public event EventHandler CanExecuteChanged;

		public void Execute(object parameter) { this.NavExecute(); }
		#endregion

		protected virtual void OnCanExecuteChanged()
		{
			if (CanExecuteChanged != null)
				CanExecuteChanged(this, new EventArgs());
		}

		void nb_Initialized(object sender, EventArgs e)
		{
			_bMapEnabled = true;
			this.NavInitialized();
			this.CanExecuteChanged(this, new EventArgs());
		}

		void nb_MapExtentUpdated(object sender, EventArgs e)
		{
			this.NavExtentUpdated();
		}

	}
	#endregion

	// Convenient base class for navigation tools
	#region NavTool
	public abstract class NavTool : NavCommand, IToggleCommand, INavTool
	{

		private string _sKey;
		private bool _bActive = false;
		private Draw _drNav = null;

		public void InitializeTool(DrawMode MyDrawMode = DrawMode.None, BitmapImage MyCursorImage = null, Point MyHotPoint = new Point())
		{

			// Register tool with ToolBehavior

			MouseCursorDef MyCursorDef = null;
			if (MyCursorImage != null)
			{
				MyCursorDef = new MouseCursorDef()
				{
					CursorImage = MyCursorImage,
					HotSpot = MyHotPoint
				};
			}
			_sKey = Guid.NewGuid().ToString();
			ToolBehavior tb = ToolBehavior.Current;
			tb.RegisterTool(_sKey, this, MyCursorDef);

			// Initialize draw

			if (MyDrawMode != DrawMode.None)
			{
				Map MyMap = MapApplication.Current.Map;
				_drNav = new Draw(MyMap);
				_drNav.DrawMode = MyDrawMode;
				_drNav.IsEnabled = false;
				_drNav.DrawComplete += new EventHandler<DrawEventArgs>(_drNav_DrawComplete);
			}

		}

		#region Overridable Methods
		abstract protected void ActivateTool();

		abstract protected void DeactivateTool();

		abstract protected void DrawComplete(Geometry g);
		#endregion

		#region NavCommand Overriddes
		protected override void NavExecute()
		{
			ToolBehavior tb = ToolBehavior.Current;
			tb.SetTool(_sKey);
		}
		#endregion

		#region IToggleCommand Implementation
		public bool IsChecked()
		{
			return _bActive;
		}
		#endregion

		#region INavTool implementation
		public void Check()
		{
			if (_drNav != null)
				_drNav.IsEnabled = true;
			this.ActivateTool();
			_bActive = true;
			this.OnCanExecuteChanged();
		}

		public void Uncheck()
		{
			if (_drNav != null)
				_drNav.IsEnabled = false;
			this.DeactivateTool();
			_bActive = false;
			this.OnCanExecuteChanged();
		}
		#endregion

		void _drNav_DrawComplete(object sender, DrawEventArgs e)
		{
			this.DrawComplete(e.Geometry);
		}

	}
	#endregion

}
