﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using System.Windows.Resources;
using System.ComponentModel.Composition;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Extensibility;
using ESRI.ArcGIS.Client.Geometry;

namespace NavAddin
{

	#region Navigation Commands
	[Export(typeof(ICommand))]
	[DisplayName("Remove Basemap")]
	[Category("Navigation Add-in")]
	[Description("Remove Basemap Command")]
	[DefaultIcon("/NavAddin.Addins;component/Images/DeleteLayer.png")]
	public class RemoveBasemapCmd : NavCommand
	{

		protected override void NavInitialized() { }

		protected override void NavExecute()
		{
			Map MyMap = MapApplication.Current.Map;
			LayerCollection MyLayers = MyMap.Layers;
			Layer BaseLayer = null;
			foreach (Layer MyLayer in MyLayers)
			{
				if (MyLayer is ArcGISTiledMapServiceLayer)
				{
					ArcGISTiledMapServiceLayer tiledLayer = (ArcGISTiledMapServiceLayer)MyLayer;
					string sUrl = tiledLayer.Url.ToLower();
					if (sUrl.IndexOf("/services.arcgisonline.com") > -1)
					{
						BaseLayer = MyLayer;
						break;
					}
				}
			}
			if (BaseLayer == null)
				return;
			MyLayers.Remove(BaseLayer);
		}

		protected override bool NavEnabled() { return true; }

		protected override void NavExtentUpdated() { }

	}

	[Export(typeof(ICommand))]
	[DisplayName("Zoom Full Extent")]
	[Category("Navigation Add-in")]
	[Description("Zoom Full Extent Command")]
	[DefaultIcon("/NavAddin.Addins;component/Images/ZoomFullExtent.png")]
	public class ZoomFullCmd : NavCommand
	{

		private Envelope _envDefault = null;

		protected override void NavInitialized()
		{
			Map MyMap = MapApplication.Current.Map;
			_envDefault= MyMap.Extent.Clone();
		}

		protected override void NavExecute()
		{
			if (_envDefault == null)
				return;
			Map MyMap = MapApplication.Current.Map;
			MyMap.ZoomTo(_envDefault);
		}

		protected override bool NavEnabled() { return true; }

		protected override void NavExtentUpdated() { }

	}

	[Export(typeof(ICommand))]
	[DisplayName("Zoom Previous")]
	[Category("Navigation Add-in")]
	[Description("Zoom Previous Command")]
	[DefaultIcon("/NavAddin.Addins;component/Images/Backward.png")]
	public class ZoomPrevCmd : NavCommand
	{

		protected override void NavInitialized() { }

		protected override void NavExecute()
		{
			NavBehavior nb = NavBehavior.Current;
			nb.ZoomPrevious();
		}

		protected override bool NavEnabled()
		{
			NavBehavior nb = NavBehavior.Current;
			return nb.CanZoomPrevious;
		}

		protected override void NavExtentUpdated()
		{
			this.OnCanExecuteChanged();
		}

	}

	[Export(typeof(ICommand))]
	[DisplayName("Zoom Next")]
	[Category("Navigation Add-in")]
	[Description("Zoom Next Command")]
	[DefaultIcon("/NavAddin.Addins;component/Images/Forward.png")]
	public class ZoomNextCmd : NavCommand
	{
	
		protected override void NavInitialized() { }

		protected override void NavExecute() 
		{
			NavBehavior nb = NavBehavior.Current;
			nb.ZoomNext();
		}

		protected override bool NavEnabled() 
		{ 
			NavBehavior nb = NavBehavior.Current;
			return nb.CanZoomNext;
		}

		protected override void NavExtentUpdated() 
		{
			this.OnCanExecuteChanged();
		}

	}

	// TO DO? Zoom In Fixed, Zoom Out Fixed
	#endregion

	#region Navigation Tools
	[Export(typeof(ICommand))]
	[DisplayName("Zoom In")]
	[Category("Navigation Add-in")]
	[Description("Zoom In Tool")]
	[DefaultIcon("/NavAddin.Addins;component/Images/ZoomIn.png")]
	public class ZoomInTool : NavTool
	{

		#region NavCommand Overrides
		protected override void NavInitialized()
		{
			string sResource = "/NavAddin.Addins;component/Images/CursorZoomIn.png";
			Point ptHotSpot = new Point(6, 6);
			Uri uri = new Uri(sResource, UriKind.Relative);
			StreamResourceInfo srInfo = Application.GetResourceStream(uri);
			BitmapImage bi = new BitmapImage();
			bi.SetSource(srInfo.Stream);
			base.InitializeTool(DrawMode.Rectangle, bi, ptHotSpot);
		}

		protected override bool NavEnabled() { return true; }

		protected override void NavExecute() { base.NavExecute(); }

		protected override void NavExtentUpdated() { }
		#endregion

		#region NavTool Overrides
		protected override void ActivateTool() { }

		protected override void DeactivateTool() { }

		protected override void DrawComplete(Geometry g)
		{
			Map MyMap = MapApplication.Current.Map;
			Envelope env = g.Extent;
			if ((env.Width == 0) && (env.Height == 0))
			{
				MapPoint pt = env.GetCenter();
				Envelope envCurrent = MyMap.Extent;
				double dW = envCurrent.Width / 4;
				double dH = envCurrent.Height / 4;
				Envelope envNew = new Envelope(pt.X - dW, pt.Y - dH, pt.X + dW, pt.Y + dH);
				envNew.SpatialReference = envCurrent.SpatialReference;
				MyMap.ZoomTo(envNew);
				return;
			}
			MyMap.ZoomTo((Envelope)g);
		}
		#endregion

	}

	[Export(typeof(ICommand))]
	[DisplayName("Zoom Out")]
	[Category("Navigation Add-in")]
	[Description("Zoom Out Tool")]
	[DefaultIcon("/NavAddin.Addins;component/Images/ZoomOut.png")]
	public class ZoomOutTool : NavTool
	{

		#region NavCommand Overrides
		protected override void NavInitialized()
		{
			string sResource = "/NavAddin.Addins;component/Images/CursorZoomOut.png";
			Point ptHotSpot = new Point(6, 6);
			Uri uri = new Uri(sResource, UriKind.Relative);
			StreamResourceInfo srInfo = Application.GetResourceStream(uri);
			BitmapImage bi = new BitmapImage();
			bi.SetSource(srInfo.Stream);
			base.InitializeTool(DrawMode.Rectangle, bi, ptHotSpot);
		}

		protected override bool NavEnabled() { return true; }

		protected override void NavExecute() { base.NavExecute(); }

		protected override void NavExtentUpdated() { }
		#endregion

		#region NavTool Overrides
		protected override void ActivateTool() { }

		protected override void DeactivateTool() { }

		protected override void DrawComplete(Geometry g)
		{
			Map MyMap = MapApplication.Current.Map;
			Envelope envCurrent = MyMap.Extent;
			Envelope env = g.Extent;
			Envelope envNew = null;
			if ((env.Width == 0) && (env.Height == 0))
			{
				MapPoint pt = env.GetCenter();
				double dW = envCurrent.Width;
				double dH = envCurrent.Height;
				envNew = new Envelope(pt.X - dW, pt.Y - dH, pt.X + dW, pt.Y + dH);
				envNew.SpatialReference = envCurrent.SpatialReference;
				MyMap.ZoomTo(envNew);
				return;
			}
			Envelope zoomBoxExtent = (Envelope)g;
			MapPoint zoomBoxCenter = zoomBoxExtent.GetCenter();
			double whRatioCurrent = envCurrent.Width / envCurrent.Height;
			double whRatioZoomBox = zoomBoxExtent.Width / zoomBoxExtent.Height;
			if (whRatioZoomBox > whRatioCurrent)
			{
				// use width
				double mapWidthPixels = MyMap.Width;
				double multiplier = envCurrent.Width / zoomBoxExtent.Width;
				double newWidthMapUnits = envCurrent.Width * multiplier;
				envNew = new Envelope(new MapPoint(zoomBoxCenter.X - (newWidthMapUnits / 2), zoomBoxCenter.Y),
											 new MapPoint(zoomBoxCenter.X + (newWidthMapUnits / 2), zoomBoxCenter.Y));
			}
			else
			{
				// use height
				double mapHeightPixels = MyMap.Height;
				double multiplier = envCurrent.Height / zoomBoxExtent.Height;
				double newHeightMapUnits = envCurrent.Height * multiplier;
				envNew = new Envelope(new MapPoint(zoomBoxCenter.X, zoomBoxCenter.Y - (newHeightMapUnits / 2)),
											 new MapPoint(zoomBoxCenter.X, zoomBoxCenter.Y + (newHeightMapUnits / 2)));
			}
			if (envNew != null)
			{
				envNew.SpatialReference = MyMap.SpatialReference;
				MyMap.ZoomTo(envNew);
			}
		}
		#endregion

	}

	[Export(typeof(ICommand))]
	[DisplayName("Pan")]
	[Category("Navigation Add-in")]
	[Description("Pan Tool")]
	[DefaultIcon("/NavAddin.Addins;component/Images/Pan.png")]
	public class PanTool : NavTool
	{

		#region NavCommand Overrides
		protected override void NavInitialized()
		{
			string sResource = "/NavAddin.Addins;component/Images/CursorPan.png";
			Point ptHotSpot = new Point(6, 6);
			Uri uri = new Uri(sResource, UriKind.Relative);
			StreamResourceInfo srInfo = Application.GetResourceStream(uri);
			BitmapImage bi = new BitmapImage();
			bi.SetSource(srInfo.Stream);
			base.InitializeTool(DrawMode.None, bi, ptHotSpot);
		}

		protected override bool NavEnabled() { return true; }

		protected override void NavExecute() { base.NavExecute(); }

		protected override void NavExtentUpdated() { }
		#endregion

		#region NavTool Overrides
		protected override void ActivateTool() { }

		protected override void DeactivateTool() { }

		protected override void DrawComplete(Geometry g) { }
		#endregion

	}
	#endregion

}
